import { getStation } from './craftingStations';
import { DefaultItems, Items, formatItemsArray, getDefaultItem } from './items';

export interface DefaultRecipe {
  id: number,
  station: string,
  inputs: Items[],
  requirements: Items[],
  outputs: Items[]
}
export type Recipe = DefaultRecipe

export const defaultRecipes: DefaultRecipe[] = [
  {
    id: -1,
    station: 'forage',
    inputs: [],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.PEBBLE),  quantity: 4 },
      { item: getDefaultItem(DefaultItems.TWIG),    quantity: 2 },
      { item: getDefaultItem(DefaultItems.BERRIES), quantity: 2 }
    ]
  },
  {
    id: -2,
    station: 'hand',
    inputs: [
      { item: getDefaultItem(DefaultItems.PEBBLE), quantity: 2 },
      { item: getDefaultItem(DefaultItems.TWIG),   quantity: 2 },
    ],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.WORKBENCH), quantity: 1 }
    ]
  },
  {
    id: -3,
    station: 'forage',
    inputs: [],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.PEBBLE),  quantity: 2 },
      { item: getDefaultItem(DefaultItems.TWIG),    quantity: 4 },
      { item: getDefaultItem(DefaultItems.APPLE),   quantity: 1 }
    ]
  },
  {
    id: -4,
    station: 'forage',
    inputs: [],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.TWIG),    quantity: 1 },
      { item: getDefaultItem(DefaultItems.COIN),    quantity: 1 },
      { item: getDefaultItem(DefaultItems.APPLE),   quantity: 4 },
      { item: getDefaultItem(DefaultItems.BERRIES), quantity: 6 },
    ]
  },
  {
    id: -5,
    station: 'forage',
    inputs: [],
    requirements: [
      { item: getDefaultItem(DefaultItems.AXE),     quantity: 1 },
    ],
    outputs: [
      { item: getDefaultItem(DefaultItems.TWIG),    quantity: 1 },
      { item: getDefaultItem(DefaultItems.LOG),     quantity: 3 },
    ]
  },
  {
    id: -6,
    station: 'workbench',
    inputs: [
      { item: getDefaultItem(DefaultItems.PEBBLE), quantity: 4 },
      { item: getDefaultItem(DefaultItems.TWIG),   quantity: 2 },
    ],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.AXE),    quantity: 1 },
    ]
  },
  {
    id: -7,
    station: 'hand',
    inputs: [],
    requirements: [
      { item: getDefaultItem(DefaultItems.AXE),     quantity: 1 },
    ],
    outputs: [
      { item: getDefaultItem(DefaultItems.BLOOD),   quantity: 6 },
    ]
  },
  {
    id: -8,
    station: 'fishing',
    inputs: [
      { item: getDefaultItem(DefaultItems.BAIT), quantity: 1 }
    ],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.CARP),        quantity: 12 },
      { item: getDefaultItem(DefaultItems.PUFFERFISH),  quantity: 4 },
      { item: getDefaultItem(DefaultItems.EXOTIC_FISH), quantity: 1 },
    ]
  },
  {
    id: -9,
    station: 'workbench',
    inputs: [
      { item: getDefaultItem(DefaultItems.TWIG), quantity: 2 },
      { item: getDefaultItem(DefaultItems.BAIT), quantity: 1 },
    ],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.FISHING_ROD), quantity: 1 }
    ]
  },
  {
    id: -10,
    station: 'forage',
    inputs: [],
    requirements: [
      { item: getDefaultItem(DefaultItems.SHOVEL), quantity: 1 },
    ],
    outputs: [
      { item: getDefaultItem(DefaultItems.BAIT),   quantity: 3 },
      { item: getDefaultItem(DefaultItems.PEBBLE), quantity: 1 },
      { item: getDefaultItem(DefaultItems.DIRT),   quantity: 1 },
    ],
  },
  {
    id: -11,
    station: 'workbench',
    inputs: [
      { item: getDefaultItem(DefaultItems.PEBBLE), quantity: 3 },
      { item: getDefaultItem(DefaultItems.TWIG),   quantity: 2 },
    ],
    requirements: [],
    outputs: [
      { item: getDefaultItem(DefaultItems.SHOVEL), quantity: 1 },
    ]
  },
];

export function getDefaultRecipe(id: number): DefaultRecipe | undefined {
  return defaultRecipes.find(recipe => recipe.id === id);
}
export function getRecipe(id: number): Recipe | undefined {
  return getDefaultRecipe(id); // currently just a stub
}

const defaultFormatRecipe = (inputs: Items[], requirements: Items[], outputs: Items[], disableBold = false) =>
  `${formatItemsArray(inputs, disableBold)}${requirements.length === 0 ? '' : ` w/ ${formatItemsArray(requirements, disableBold)}`} => ${formatItemsArray(outputs, disableBold)}`;

export function formatRecipe(recipe: DefaultRecipe, disableBold = false) {
  const station = getStation(recipe.station);
  return (station?.formatRecipe || defaultFormatRecipe)(recipe.inputs, recipe.requirements, recipe.outputs, disableBold);
}